/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.ajax4jsf.bean;

import java.lang.reflect.Method;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import javax.faces.convert.DateTimeConverter;
import javax.faces.event.ValueChangeEvent;

import org.richfaces.component.UICalendar;
import org.richfaces.event.CurrentDateChangeEvent;

public class CalendarTestBean {

    public static final String valueChangeListener = "valueChangeListener";

    public static final String currentDateChangeListener = "currentDateChangeListener";

    public static final String DATE_PATTERN = "MM/dd/yyyy HH:mm";

    public static final Locale LOCALE = Locale.US;

    public static final TimeZone TIME_ZONE = TimeZone.getTimeZone("GMT+2");

    public static Date DEFAULT_DATE;

    private static DateTimeConverter DEFAULT_CONVERTER;

    public static DateFormat DATE_FORMAT;

    // constructor for static fields
    static {
        setupDefaultConverter();
        setupDefaultDate();
    }

    private String datePattern;

    private Locale locale;

    private TimeZone timeZone;

    private Date selectedDate;

    private String selectedDateString;

    private CalendarDataModel model;

    private String mode = UICalendar.AJAX_MODE;

    private String status;
    
    private Date preloadDateRangeStart;
    
    private Date preloadDateRangeEnd;
    

    public CalendarTestBean() {
        selectedDate = DEFAULT_DATE;
        resetSelectedDateString();

        datePattern = DATE_PATTERN;
        locale = LOCALE;
        timeZone = TIME_ZONE;
        Calendar c = getLocale()!=null ? Calendar.getInstance(getLocale()) : Calendar.getInstance();
        c.set(c.get(Calendar.YEAR), c.get(Calendar.MONTH) - 1, 1); 
        preloadDateRangeStart = c.getTime(); // Set preloadBegin by the first day of the last month
        c.set(c.get(Calendar.YEAR), c.get(Calendar.MONTH) + 3, 0);
        preloadDateRangeEnd = c.getTime(); // Set preloadEnd by the last day of the next month
    }

    public Date getSelectedDate() {
        return selectedDate;
    }

    public void setSelectedDate(Date selectedDate) {
        this.selectedDate = selectedDate;
        resetSelectedDateString();
    }

    public void resetSelectedDateString() {
        if (getSelectedDate() != null) {
            setSelectedDateString(DATE_FORMAT.format(getSelectedDate()));
        } else {
            setSelectedDateString("");
        }
    }

    public String getSelectedDateString() {
        return selectedDateString;
    }

    public void setSelectedDateString(String selectedDateString) {
        this.selectedDateString = selectedDateString;
    }

    public String getDatePattern() {
        return datePattern;
    }

    public Locale getLocale() {
        return locale;
    }

    public TimeZone getTimeZone() {
        return timeZone;
    }

    public void setDatePattern(String datePattern) {
        this.datePattern = datePattern;
    }

    public void setLocale(Locale locale) {
        this.locale = locale;
    }

    public void setTimeZone(TimeZone timeZone) {
        this.timeZone = timeZone;
    }

    public static Calendar getCalendar() {
        return Calendar.getInstance(CalendarTestBean.TIME_ZONE, CalendarTestBean.LOCALE);
    }

    /**
     * Setup the default date
     */
    private static void setupDefaultDate() {
        DEFAULT_DATE = getDayInMay(10);
    }

    public static Date getDayInMay(int dayOfMonth) {
        Calendar calendar = getCalendar();
        calendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
        calendar.set(Calendar.MONTH, Calendar.MAY);
        calendar.set(Calendar.YEAR, 2008);
        calendar.set(Calendar.HOUR_OF_DAY, 18);
        calendar.set(Calendar.MINUTE, 25);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    /**
     * Setup the default converter
     */
    private static void setupDefaultConverter() {
        DEFAULT_CONVERTER = new DateTimeConverter();

        DEFAULT_CONVERTER.setPattern(DATE_PATTERN);
        DEFAULT_CONVERTER.setLocale(LOCALE);
        DEFAULT_CONVERTER.setTimeZone(TIME_ZONE);

        try {
            Method method = null;
            Method[] declaredMethods = DateTimeConverter.class.getDeclaredMethods();
            for (int i = 0; i < declaredMethods.length; i++) {
                if (declaredMethods[i].getName().equals("getDateFormat")) {
                    method = declaredMethods[i];
                    break;
                }
            }
            if (method != null) {

                method.setAccessible(true);
                DATE_FORMAT = (DateFormat) method.invoke(DEFAULT_CONVERTER, DEFAULT_CONVERTER.getLocale());
                DATE_FORMAT.setTimeZone(DEFAULT_CONVERTER.getTimeZone());
            }
        } catch (Exception e) {
            // skip exception
        }
    }

    public void valueChangeListener(ValueChangeEvent event) {
        status = getStatus() + valueChangeListener;
    }

    public void currectDateChangeListener(CurrentDateChangeEvent event) {
        status = getStatus() + currentDateChangeListener;
    }

    public String testClientMode() {
        mode = UICalendar.CLIENT_MODE;
        return null;
    }

    public void reset() {
        mode = UICalendar.AJAX_MODE;
        status = "";
        selectedDate = new Date();
        isPopup = false;
        currentDate = new Date();
    }

    public String resetAction() {
        reset();
        return null;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getMode() {
        return mode;
    }

    public void setMode(String mode) {
        this.mode = mode;
    }

    public CalendarDataModel getModel() {
        return model;
    }

    public void setModel(CalendarDataModel model) {
        this.model = model;
    }

    private Date currentDate = null;

    /**
     * Gets value of currentDate field.
     * 
     * @return value of currentDate field
     */
    public Date getCurrentDate() {
        return currentDate;
    }

    /**
     * Set a new value for currentDate field.
     * 
     * @param currentDate
     *                a new value for currentDate field
     */
    public void setCurrentDate(Date currentDate) {
        this.currentDate = currentDate;
    }

    private boolean isPopup;

    /**
     * Gets value of isPopup field.
     * @return value of isPopup field
     */
    public boolean isPopup() {
        return isPopup;
    }

    /**
     * Set a new value for isPopup field.
     * @param isPopup a new value for isPopup field
     */
    public void setPopup(boolean isPopup) {
        this.isPopup = isPopup;
    }

	/**
	 * @return the preloadDateRangeStart
	 */
	public Date getPreloadDateRangeStart() {
		return preloadDateRangeStart;
	}

	/**
	 * @param preloadDateRangeStart the preloadDateRangeStart to set
	 */
	public void setPreloadDateRangeStart(Date preloadDateRangeStart) {
		this.preloadDateRangeStart = preloadDateRangeStart;
	}

	/**
	 * @return the preloadDateRangeEnd
	 */
	public Date getPreloadDateRangeEnd() {
		return preloadDateRangeEnd;
	}

	/**
	 * @param preloadDateRangeEnd the preloadDateRangeEnd to set
	 */
	public void setPreloadDateRangeEnd(Date preloadDateRangeEnd) {
		this.preloadDateRangeEnd = preloadDateRangeEnd;
	}

}
